/* 
 * ROPStop is Copyright (c) 2012-2014 Barton P. Miller. Written by Emily R. Jacobson.
 *
 * This file is part of ROPStop.
 *
 * ROPStop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * ROPStop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ROPStop.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Includes */
#include <stdio.h>
#include <iostream>
#include <sstream>
#include <string.h>

/* ProcControl includes */
#include "PCProcess.h"
#include "PCErrors.h"
#include "Event.h"

/* Stackwalker includes */
#include "walker.h"
#include "steppergroup.h"
#include "framestepper.h"
#include "swk_errors.h"

#include "generateWalker.h"
#include "globals.h"

using namespace std;
using namespace Dyninst;

Stackwalker::Walker * generateAnalysisWalker(Stackwalker::Walker * walker)
{
    /* Create static analysis-only stepper group */
    Stackwalker::Walker * analysisWalker = 
        Stackwalker::Walker::newWalker(walker->getProcessState(), NULL, NULL, false);
    if (!analysisWalker) {
        if (DEBUG) fprintf(stderr, "newWalker failed: %s\n", Stackwalker::getLastErrorMsg());
        return NULL;
    } 

    Stackwalker::StepperGroup * steppergroup = walker->getStepperGroup();
    std::set<Stackwalker::FrameStepper *> steppers;
    steppergroup->getSteppers(steppers);
    std::set<Stackwalker::FrameStepper *>::iterator sgIter;
    for (sgIter = steppers.begin();
            sgIter != steppers.end();
            ++sgIter) {
        if ( strcmp((*sgIter)->getName(), "AnalysisStepper") == 0) {
            analysisWalker->addStepper(*sgIter);
        } else if ( strcmp((*sgIter)->getName(), "BottomOfStackStepper") == 0) {
            analysisWalker->addStepper(*sgIter);
        } else if ( strcmp((*sgIter)->getName(), "SigHandlerStepper") == 0) {
            analysisWalker->addStepper(*sgIter);
        } else if ( strcmp((*sgIter)->getName(), "DebugStepper") == 0) {
            analysisWalker->addStepper(*sgIter);
        }
    }
    return analysisWalker;
}

Stackwalker::Walker * generateWalker(pid_t pid, std::string file, bool analysis)
{
    Stackwalker::Walker * walker = Stackwalker::Walker::newWalker(pid, file);
    if (!walker) {
        if (DEBUG) fprintf(stderr, "newWalker failed: %s\n", Stackwalker::getLastErrorMsg());
        return NULL;
    }

    if (analysis) {
        return generateAnalysisWalker(walker);
    } else {
        return walker;
    }
}

Stackwalker::Walker * generateWalker(ProcControlAPI::Process::ptr proc, 
        bool analysis)
{
    Stackwalker::Walker * walker = Stackwalker::Walker::newWalker(proc);
    if (!walker) {
        if (DEBUG) fprintf(stderr, "newWalker failed: %s\n", Stackwalker::getLastErrorMsg());
        return NULL;
    }

    if (analysis) {
        return generateAnalysisWalker(walker);
   } else {
       return walker;
   }
}
