/* 
 * ROPStop is Copyright (c) 2012-2014 Barton P. Miller. Written by Emily R. Jacobson.
 *
 * This file is part of ROPStop.
 *
 * ROPStop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * ROPStop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ROPStop.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __GLOBALS_H__
#define __GLOBALS_H__

#include <tr1/unordered_map>
#include "PCProcess.h"
#include "walker.h"
#include "CodeObject.h"
#include "CodeSource.h"
#include "Instruction.h"

using namespace std;
using namespace Dyninst;

/* Debugging option: if false, turn off validation */
#define CHECKER true

/* Debugging option: if true, continue running process even if an invalid state is detected */
#define CONTINUE false

extern ProcControlAPI::Process::ptr NEW_PROCESS;
extern ProcControlAPI::Process::ptr NEW_EXEC;
extern ProcControlAPI::Thread::ptr NEW_THREAD;
typedef std::pair<ProcControlAPI::Process::ptr, Stackwalker::Walker *> PW_pair;
extern std::set<PW_pair> monitored;
extern int COUNTER;
extern std::tr1::unordered_map<string, ParseAPI::SymtabCodeSource*> srcs;
extern std::tr1::unordered_map<string, ParseAPI::CodeObject*> objs;
extern Architecture arch;
extern std::map<ParseAPI::CodeObject*, std::map<Address, ParseAPI::Function*> > preproc;
extern std::tr1::unordered_map<Address, std::pair<std::string, Address> > cachedLibs;
extern std::tr1::unordered_map<Address, ParseAPI::Function*> framefuncs;

class callLocation {
    private:
        Address a;
        ParseAPI::Function * f;
        InstructionAPI::Instruction::Ptr i;
    public:
        callLocation() {}

        callLocation(Address _a, ParseAPI::Function * _f, InstructionAPI::Instruction::Ptr _i) :
            a(_a), f(_f), i(_i) {}

        Address addr() { return a; }
        ParseAPI::Function * func() { return f; }
        InstructionAPI::Instruction::Ptr instr() { return i; }
};

/* Input globals */
extern bool ANALYSIS;
extern bool ALL_SYSCALLS;
extern bool VERBOSE;
extern bool LITERAL;
extern bool ORIG_INSTR;
extern bool DEBUG;

extern int COUNTER;

extern MachRegister EAX;

#endif
