/* 
 * ROPStop is Copyright (c) 2012-2014 Barton P. Miller. Written by Emily R. Jacobson.
 *
 * This file is part of ROPStop.
 *
 * ROPStop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * ROPStop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ROPStop.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdio.h"
#include "accessors.h"

#include "assert.h"

using namespace std;
using namespace Dyninst;

MachRegister EAX;

bool loadKnownLibraries(char * BIN_FILE)
{
   std::vector<std::string> libs;
   if (BIN_FILE) { libs.push_back(BIN_FILE); }
//   libs.push_back("/usr/lib/libstdc++.so.6");
//   libs.push_back("/lib/libgcc_s.so.1");
//   libs.push_back("/lib/libc-2.12.so");
//   libs.push_back("/lib/ld-2.12.so");

   for (std::vector<std::string>::iterator iter = libs.begin(); 
           iter != libs.end(); ++iter) {
       ParseAPI::CodeObject* co = getCodeObject(NULL, *iter, 0, 0);
       if (!co) {
           fprintf(stderr, "\t loadKnownLibraries: getCodeObject(%s) failed\n", iter->c_str());
           return false;
       }
       // Set some globals based on the arch of our executable
       ParseAPI::CodeSource* cs = co->cs(); 
       ParseAPI::SymtabCodeSource* scs = dynamic_cast<ParseAPI::SymtabCodeSource*>(cs);
        if (scs->getArch() == Arch_x86) {
            EAX = x86::oeax;
        } else if (scs->getArch() == Arch_x86_64) { 
            EAX = x86_64::orax;
        } else {
            assert(0 && "ROPStop cannot handle non-x86 architectures currently.");
        }
   }

    return true;
}
