/* 
 * ROPStop is Copyright (c) 2012-2014 Barton P. Miller. Written by Emily R. Jacobson.
 *
 * This file is part of ROPStop.
 *
 * ROPStop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * ROPStop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ROPStop.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Includes */
#include <stdio.h>
#include <getopt.h>
#include <iostream>
#include <sstream>

#include "dyntypes.h"

#include <sys/time.h>
#include <sys/resource.h>

/* ProcControl includes */
#include "PCProcess.h"
#include "PCErrors.h"

/* Stackwalker includes */
#include "walker.h"
#include "frame.h"
#include "basetypes.h"
#include "framestepper.h"
#include "steppergroup.h"
#include "swk_errors.h"
#include "procstate.h"

/* InstructionAPI includes */
#include "Instruction.h"
#include "InstructionDecoder.h"

#include "CFG.h"
#include "CodeObject.h"
#include "CodeSource.h"

/* local includes */
#include "manualWalk.h"

#include "printers.h"
#include "accessors.h"

using namespace std;
using namespace Dyninst;


void printStepperGroup(Stackwalker::StepperGroup * steppergroup)
{
    cout << "walker created with StepperGroup: " << endl;
    std::set<Stackwalker::FrameStepper *> steppers;
    steppergroup->getSteppers(steppers);
    std::set<Stackwalker::FrameStepper *>::iterator sgIter;
    for (sgIter = steppers.begin();
            sgIter != steppers.end();
            ++sgIter) {
        cout << "\t " << (*sgIter)->getName() << ", priority " << std::hex << (*sgIter)->getPriority() << endl;
    }
}

std::string printFrameInfo(Stackwalker::Frame & frame, bool full)
{
    ParseAPI::Function * func = getFrameFunc(frame.getWalker(), frame.getRA());
    string name = "";
    if (func) {
        name = func->name();
    }

    stringstream ret; 
    ret << "frame: \"" << name << "\"";

    if (frame.getStepper()) {
        ret << " [" << frame.getStepper()->getName() << "] ";
    }


    ret << ", RA = 0x" << std::hex << frame.getRA();

    Stackwalker::location_t raloc = frame.getRALocation();
    ret << " @ " << raloc.format() << endl;
#if 0 
    if (raloc.location == Stackwalker::loc_address) {
        ret << " @ " << std::hex << raloc.val.addr;
    } else if (raloc.location == Stackwalker::loc_register) {
        ret << " @ " << std::hex << raloc.val.reg.name();
    } else {
        ret << " @ unknown";
    }
#endif

    if (full) {
        ret << ", FP = 0x" << std::hex << frame.getFP();
#if 0      
  	Stackwalker::location_t fploc = frame.getFPLocation();
        if (fploc.location == Stackwalker::loc_address) {
            ret << " @ " << std::hex << fploc.val.addr;
        } else if (fploc.location == Stackwalker::loc_register) {
            ret << " @ " << std::hex << fploc.val.reg.name();
        } else {
            ret << " @ unknown";
        }
#endif

        ret << ", SP = 0x" << std::hex << frame.getSP();
#if 0
	  Stackwalker::location_t sploc = frame.getSPLocation();
        if (sploc.location == Stackwalker::loc_address) {
            ret << " @ " << std::hex << sploc.val.addr;
        } else if (sploc.location == Stackwalker::loc_register) {
            ret << " @ " << std::hex << sploc.val.reg.name();
        } else {
            ret << " @ unknown";
        }
#endif
    }

    std::string lib;
    Dyninst::Offset offset;
    void * symtab;
    if (frame.getLibOffset(lib, offset, symtab)) {
        ret << " [in " << lib << "]";
    }


    return ret.str();
}

void printRusage()
{

    struct rusage* usage = (struct rusage*)malloc(sizeof(struct rusage));
    assert(usage);
    if (getrusage(RUSAGE_SELF, usage) != 0) {
        perror("printRusage: getrusage");
        return;
    }
    
    fprintf(stderr, "rusage information:\n");
    struct timeval ru_utime = usage->ru_utime;
    struct timeval ru_stime = usage->ru_stime;
    double u = (ru_utime.tv_sec + ru_utime.tv_usec/1.0e6);
    double s = (ru_stime.tv_sec + ru_stime.tv_usec/1.0e6);
    fprintf(stderr, "\t u: %f, s: %f\n", u, s);

    long ru_maxrss = usage->ru_maxrss;
    fprintf(stderr, "\t maxrss: %ld\n", ru_maxrss);
    
    return;
}
