/* 
 * ROPStop is Copyright (c) 2012-2014 Barton P. Miller. Written by Emily R. Jacobson.
 *
 * This file is part of ROPStop.
 *
 * ROPStop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * ROPStop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ROPStop.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>

#include "PCProcess.h"
#include "PCErrors.h"

#include "walker.h"

#include "setupProcess.h"
#include "callbacks.h"
#include "generateWalker.h"
#include "globals.h"
#include "accessors.h"
#include "printers.h"

using namespace std;
using namespace Dyninst;

bool setupProcess(ProcControlAPI::Process::ptr proc, 
        Stackwalker::Walker ** walker, 
        bool analysis)
{

    if (DEBUG) fprintf(stderr, "Calling setupProcess for %d\n", proc->getPid());

    /* Set up callbacks */
    addCallbacks(proc);   

    /* Create a new stackwalker */
    *walker = generateWalker(proc, analysis);
    if (!walker) {
        return false;
    }

    /* Set threads to use syscall mode */
    ProcControlAPI::ThreadPool & threads = proc->threads();
    ProcControlAPI::ThreadPool::iterator tIter;

    for (tIter = threads.begin(); tIter != threads.end(); ++tIter) {     
        ProcControlAPI::Thread::ptr thread = *tIter;
        thread->setSyscallMode(true);
    }

    /* Parse binary */
    /* Get executable */
    ProcControlAPI::LibraryPool & libraries = proc->libraries();
    ParseAPI::CodeObject * co = NULL;
    std::string libName;
    Address libLoad;
    unsigned libSize = -1;
    ProcControlAPI::Library::ptr library;
    ProcControlAPI::LibraryPool::iterator lIter;
    for (lIter = libraries.begin(); lIter != libraries.end(); ++lIter) {
        library = *lIter;
        libName = library->getName();

        if (DEBUG) fprintf(stderr, "setupProcess: parsing %s\n", libName.c_str());

        libLoad = library->getLoadAddress();

        // getCodeObject will trigger a full parse of the object
        co = getCodeObject(*walker, libName, libLoad, libSize);
        if (!co) {
            if (DEBUG) fprintf(stderr, "setupProcess: getCodeObject failed: %s\n", libName.c_str());
            continue;
        }
    }

    
    /* Continue execution */
    fprintf(stderr, "setupProcess complete; process %d continuing------------------\n", proc->getPid());
    if (!proc->continueProc()) {
        if (DEBUG) fprintf(stderr, "proc->continueProc failed: %s\n", ProcControlAPI::getLastErrorMsg());
        return false;
    }

    return true;
}
